#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>
// ESP8266 Software SPI (slower updates, more flexible pin options):
// pin 14 - Serial clock out (SCLK)
// pin 13 - Serial data out (DIN)
// pin 12 - Data/Command select (D/C)
// pin 15 - LCD chip select (CS)
// pin 4 - LCD reset (RST)
Adafruit_PCD8544 display = Adafruit_PCD8544(14, 13, 12, 15, 4);

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <IPAddress.h>
#include <WiFiUdp.h>
#include <stdio.h>
#include <inttypes.h>

 
const char* ssid = "ardrone2_060602";
const int navPort = 5554;
const int atPort = 5556;
const IPAddress drone(192, 168, 1, 1);

byte pos;
unsigned int sequence;
unsigned int lastNav;
unsigned int lastPacket;
WiFiUDP Udp;
WiFiUDP AT;
String sendBuffer;
char incoming[1024];


void setup(void) {
  Serial.begin(115200);
  Serial.println("");
  Serial.println("Starting");
  // initialize the LCD
  display.begin();
  display.setContrast(50);
  display.display(); // show splashscreen
  delay(2000);
  display.clearDisplay();   // clears the screen and buffer
  display.setTextSize(1);
  display.setTextColor(BLACK);

  // Turn on the blacklight and print a message.
  display.setCursor(0,0);
  display.print("WiFi connect ...");
  display.display();
  
  pos = 0;
  sequence = 1;
  // Connect to WiFi network
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid);
  
 
  // Wait for connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(200);
  }
  Serial.println("Connected!");
  Serial.println(WiFi.localIP());
  display.clearDisplay();
  display.setCursor(0,0);
  display.print("OK");
  delay(3000);
  display.setCursor(0,20);
  display.print(WiFi.localIP());
  display.display();
  //
  pinMode(pinButton,INPUT);
  //  
  Udp.begin(navPort); //Open port for navdata
  Udp.flush();
  AT.begin(atPort);
  AT.flush();
  String configg = "AT*CONFIG=";
  configg += String(sequence);
  configg += ",\"general:navdata_demo\",\"TRUE\"\r";
  while(Udp.parsePacket() == 0) {
    delay(10);
    Udp.beginPacket(drone, navPort);
    Udp.write(0x01);
    Udp.endPacket();
    delay(10);
    sendPacket(configg);
  }
  Serial.println("Starting main loop"); 
  //delay(3000);

  
}

void loop(void) {
  if(Udp.parsePacket()) {
    int len = Udp.read(incoming, 1024);
    Serial.print("length=");Serial.println(len);
    if (len <  30) return;
    incoming[len] = 0;
    Serial.print("header=");printParamData(0,4);Serial.println();
    Serial.print("state=");printParamData(4,4);Serial.println();
    Serial.print("pitch=");printParamData(28,1);Serial.println();
    Serial.print("roll=");printParamData(32,1);Serial.println();    
    Serial.print("yaw=");printParamData(36,1);Serial.println();
    Serial.print("altitude=");printParamData(40,1);Serial.println();
    Serial.print("battery=");printParamData(24,1);Serial.println();    
    Serial.print("vx=");printParamData(44,1);Serial.println();
    Serial.print("vy=");printParamData(52,1);Serial.println();       
    Serial.println("********************************************");
    // печать параметров на дисплей
    printdatalcd();
  }
  // отправка пакета для поддержания соединения
  if(millis() - lastPacket > 1000) {
    String tmr = "AT*COMWDG=";
    tmr += String(sequence);
    sendPacket(tmr);
    Serial.print("send=");Serial.println(tmr);
  }
}
// отправка в порт 5554
void sendPacket(String &string) {
 char sendChar[string.length()+1];
 string.toCharArray(sendChar, string.length()+1);
 sendChar[string.length()] = '\r';
 AT.beginPacket(drone, atPort);
 AT.write(sendChar);
 AT.endPacket();
 sequence++;
 lastPacket = millis();
}
// печать данных в последовательный порт
void printParamData(int offset,int count) {
    for(int i=count;i>0;i--) {
      Serial.print(incoming[offset+i-1],HEX);Serial.print(" ");
    }
}
// данные на экране lcd
void printdatalcd() {
    // status
    display.clearDisplay();    
    display.setCursor(0,0);
    display.print(incoming[4],HEX);display.print(" ");
    display.print(incoming[5],HEX);display.print(" ");
    display.print(incoming[6],HEX);display.print(" ");
    display.print(incoming[7],HEX);display.print(" ");
    // battery
    display.setCursor(0,20);
    display.print(incoming[24],DEC);
    display.print("%");
    // altitude h
    display.setCursor(0,40);
    display.print(incoming[40],HEX);display.print(" ");
    // vx
    display.print(incoming[40],HEX);display.print(" ");
    // vy
    display.print(incoming[40],HEX);display.print(" ");
    // vz
    display.print(incoming[40],HEX);display.print(" ");  
    display.display();      
}

